--[[
Wardrobe Plus
Allows adding new Designs to the Ingame Wardrobe

Copyright (C) [F/A]braeven

Author: braeven
Date: 09.05.2023
Version: 1.0.0.0

Contact/Help
https://github.com/braeven/FS22_Production_Revamp
Or Production Revamp Discord / Maize Plus Discord


Changelog:
0.1.0.0 @ 11.04.2023 - First Test
0.2.0.0 @ 15.04.2023 - Added Female Variants, fix initial bug that required switching model to view new clothes
0.3.0.0 @ 16.04.2023 - Added Multiple Instances Support
0.4.0.0 @ 21.04.2023 - PlayerStyle saved in the GameSettings should be reloaded
0.5.0.0 @ 22.04.2023 - Reduced Filesizes, made overview-icons change between male/female
0.6.0.0 @ 28.04.2023 - Major rewrite
0.7.0.0 @ 29.04.2023 - Some Fixes, Rename
1.0.0.0 @ 09.05.2023 - Release Version
1.0.0.1 @ 02.06.2023 - add topAviatorJacket


Important:
No changes are allowed to this script without permission from Braeven.
Don't copy the script into a mod/map, load this mod as a dependency!

Es dürfen an diesem Script keine Veränderungen ohne Erlaubnis von Braeven gemacht werden.
Nicht das Script in mods/maps kopieren, ladet den Mod über eine Dependency!
]]





FAMerchStore = {}

PlayerStyle.FAMerchStore = {}
PlayerStyle.FAMerchStore.maxUV = 300
PlayerStyle.FAMerchStore.diffuses = {}
PlayerStyle.FAMerchStore.masks = {}
PlayerStyle.FAMerchStore.icons = {}
PlayerStyle.FAMerchStore.modName = {}
PlayerStyle.FAMerchStore.male = {}
PlayerStyle.FAMerchStore.female = {}	
PlayerStyle.FAMerchStore.type = {}



--XML Schema erstellen
function FAMerchStore:AddXmlSchema()
	FAMerchStore.XmlSchema = XMLSchema.new("FAMerchStore")
	local schema = FAMerchStore.XmlSchema
	schema:register(XMLValueType.STRING, "modDesc.clothes.item(?)", "FA Merch Store Script", nil, false)
	schema:register(XMLValueType.STRING, "modDesc.clothes.item(?)#diffuse", "FA Merch Store: Path to the diffuse-file", nil, false)
	schema:register(XMLValueType.STRING, "modDesc.clothes.item(?)#mask", "FA Merch Store: Path to the mask-file", nil, false)
	schema:register(XMLValueType.STRING, "modDesc.clothes.item(?)#type", "FA Merch Store: Clothing-Type", nil, false)
	schema:register(XMLValueType.STRING, "modDesc.clothes.item(?)#icon", "FA Merch Store: Icon for the clothing", nil, false)
end

FAMerchStore:AddXmlSchema()



--Neue Kleidung in Tabellen hinterlegen
function PlayerStyle.FAMerchStore:loadClothes(modDir, modName)
	local xmlFile = XMLFile.load("FAMerchStore", modDir.."modDesc.xml", FAMerchStore.XmlSchema)

	if xmlFile ~= nil and xmlFile ~= 0 then
		print("Wardrobe Plus: Loading Clothes for " .. modName)
		local before = {}

		xmlFile:iterate("modDesc.clothes.item", function (_, clothesKey)
			local clothesType = string.upper(xmlFile:getValue(clothesKey .. "#type"))
			local mask = xmlFile:getString(clothesKey .. "#mask")
			local diffuse = xmlFile:getString(clothesKey .. "#diffuse")
			local icon = xmlFile:getString(clothesKey .. "#icon")

			if clothesType == "VINTAGE" or clothesType == "MILITIA" or clothesType == "BEANIE" or clothesType == "BALLCAP" or clothesType == "HOODIE" or clothesType == "SWEATER" or clothesType == "POLOSHIRT" or clothesType == "TSHIRTF" or clothesType == "TSHIRTM" or clothesType == "AVIATOR" then
				if fileExists(modDir .. mask) and fileExists(modDir .. diffuse) and fileExists(modDir .. icon) then
					local maxUV = PlayerStyle.FAMerchStore.maxUV + 1
					PlayerStyle.FAMerchStore.masks[maxUV] = modDir .. mask
					PlayerStyle.FAMerchStore.diffuses[maxUV] = modDir .. diffuse
					PlayerStyle.FAMerchStore.icons[maxUV] = modDir .. icon
					PlayerStyle.FAMerchStore.female[maxUV] = true
					PlayerStyle.FAMerchStore.male[maxUV] = true
					PlayerStyle.FAMerchStore.type[maxUV] = clothesType
					if before[clothesType] == nil then
						PlayerStyle.FAMerchStore.modName[maxUV] = modName .. "_0"
						before[clothesType] = 1
					else
						PlayerStyle.FAMerchStore.modName[maxUV] = modName .."_".. before[clothesType]
						before[clothesType] = before[clothesType] + 1
					end
					if clothesType == "TSHIRTF" then
						PlayerStyle.FAMerchStore.male[maxUV] = false
					elseif clothesType == "TSHIRTM" then
						PlayerStyle.FAMerchStore.female[maxUV] = false
					end
					PlayerStyle.FAMerchStore.maxUV = maxUV
				else
					print("Error Wardrobe Plus: ".. modName .." - mask, diffuse or icon not found for : '" .. clothesType .. "', skipped adding.")
				end
			else
				print("Error Wardrobe Plus: ".. modName .." - Invalid Clothing-Type: '" .. clothesType .. "', skipped adding.")
			end
		end)
	end

	xmlFile:delete()
end



--Überschrieben um Texturen austauschen zu können
function FAMerchStore:apply(superFunc, skeleton, mesh, hideBody, models)
	self:loadConfigurationIfRequired()

	if #self.faceConfig.items == 0 then
		return
	end

	local function indexToObject(index)
		if index == nil then
			return nil
		end

		if index.isSkeleton then
			return I3DUtil.indexToObject(skeleton, index.path)
		else
			return I3DUtil.indexToObject(mesh, index.path)
		end
	end

	local function getNodeClone(filename)
		if models[filename] == nil then
			return nil
		end

		local cloneId = clone(models[filename], false, false, false)

		return cloneId
	end

	for _, nodeIndex in pairs(self.attachPoints) do
		local node = indexToObject(nodeIndex)

		for i = getNumOfChildren(node) - 1, 0, -1 do
			delete(getChildAt(node, i))
		end
	end

	if self.faceConfig.selection ~= 0 then
		local skinColorR, skinColorG, skinColorB = unpack(self.faceConfig.items[self.faceConfig.selection].skinColor)

		for _, part in ipairs(self.bodyParts) do
			local node = indexToObject(part.index)

			setVisibility(node, not hideBody)
			setShaderParameter(node, "colorScaleR", skinColorR, skinColorG, skinColorB, 1, false)
		end

		local selectedFace = self.faceConfig.items[math.max(self.faceConfig.selection, 1)]
		local modelNode = getNodeClone(selectedFace.filename)

		if modelNode == nil then
			print_r(models)
			printCallstack()

			return
		end

		local faceSourceNode = I3DUtil.indexToObject(modelNode, selectedFace.index)
		local oldFaceSkeleton = getChildAt(modelNode, 0)

		link(indexToObject(selectedFace.attachNode), faceSourceNode)

		for i = 0, getNumOfChildren(faceSourceNode) - 1 do
			local node = getChildAt(faceSourceNode, i)

			setShapeBones(node, skeleton, oldFaceSkeleton, true)

			if getHasShaderParameter(node, "sssColor") then
				setShaderParameter(node, "colorScaleR", skinColorR, skinColorG, skinColorB, 1, false)
			end
		end

		delete(modelNode)
	end

	local function enable(item, extentToCompare, extentToCompare2)
		if item == nil then
			return nil
		end

		local modelNode = getNodeClone(item.filename)

		if modelNode == nil then
			return nil
		end

		local node = I3DUtil.indexToObject(modelNode, item.index)
		local attachNode = indexToObject(item.attachNode)

		link(attachNode, node)

		local node2 = nil

		if item.index2 ~= nil then
			node2 = I3DUtil.indexToObject(modelNode, item.index2)

			link(attachNode, node2)
		end

		if not item.attachNode.isSkeleton then
			local oldSkeleton = getChildAt(modelNode, 0)

			I3DUtil.setShapeBonesRec(node, skeleton, oldSkeleton, true)

			if node2 ~= nil then
				I3DUtil.setShapeBonesRec(node2, skeleton, oldSkeleton, true)
			end
		end

		for _, partIndex in ipairs(item.hiddenBodyParts) do
			local partNode = indexToObject(self.bodyParts[partIndex].index)

			setVisibility(partNode, false)
		end

		if extentToCompare ~= nil and item.extents ~= nil then
			if item.extents[extentToCompare] == nil then
				extentToCompare = "hands"
			end

			local extentsSeen = {}

			for typ, nodeIndex in pairs(item.extents) do
				local show = false

				if typ == extentToCompare or typ == extentToCompare2 then
					show = true
				end

				local extentNode = I3DUtil.indexToObject(node, nodeIndex)

				if extentsSeen[extentNode] == nil then
					setVisibility(extentNode, show)

					if show then
						extentsSeen[extentNode] = true
					end
				end
			end
		end

		if item.transforms ~= nil then
			local transform = item.transforms[self.faceConfig.selection]

			setTranslation(node, unpack(transform.translation))
			setRotation(node, math.rad(transform.rotation[1]), math.rad(transform.rotation[2]), math.rad(transform.rotation[3]))
			setScale(node, unpack(transform.scale))
		end

		delete(modelNode)

		return node, node2
	end

	local function updateColors(node, item, selectedColorIndex)
		if node == nil then
			return
		end

		local numColors = item.numColors

		if numColors == 0 then
			return
		end

		if selectedColorIndex == nil then
			printCallstack()

			return
		end

		local color = item.colors[selectedColorIndex]

		if color ~= nil and numColors >= 1 then
			local primaryColor = color.primary

			I3DUtil.setShaderParameterRec(node, "colorScaleR", primaryColor[1], primaryColor[2], primaryColor[3], primaryColor[4], false)
		end

		if color ~= nil and numColors >= 2 then
			local secondaryColor = color.secondary

			if secondaryColor ~= nil then
				I3DUtil.setShaderParameterRec(node, "colorScaleG", secondaryColor[1], secondaryColor[2], secondaryColor[3], secondaryColor[4], false)
			end
		end
	end

	local function updateHairColors(node, item)
		if item == nil or node == nil then
			return
		end

		local selectedColorItem = item.colors[self.hairStyleConfig.color]
		local hairColorR, hairColorG, hairColorB = unpack(selectedColorItem.primary)
		local hairColorR2, hairColorG2, hairColorB2 = unpack(selectedColorItem.secondary)

		I3DUtil.setShaderParameterRec(node, "colorScaleG", hairColorR, hairColorG, hairColorB, 1, false)
		I3DUtil.setShaderParameterRec(node, "colorScaleR", hairColorR2, hairColorG2, hairColorB2, 1, false)
	end

	if self.topConfig.selection ~= 0 then
		local itemAttachNode = enable(self.topConfig.items[self.topConfig.selection], self.glovesConfig.selection ~= 0 and self.glovesConfig.items[self.glovesConfig.selection].extent or "hands")
		local top = self.topConfig.items[self.topConfig.selection]
		if top.uvSlot > 300 then
			local overwrite = ""
			local sleeves1 = ""
			local sleeves2 = ""
			local sleeves3 = ""
			if top.filename == "dataS/character/player02/clothes/tops/topLightSweater.i3d" or top.filename == "dataS/character/player01/clothes/tops/topLightSweater.i3d" then
				overwrite = I3DUtil.getChildByName(itemAttachNode, "lightSweater")
				sleeves1 = I3DUtil.getChildByName(itemAttachNode, "sleevesHands")
				sleeves2 = I3DUtil.getChildByName(itemAttachNode, "sleevesOverGlove")
				sleeves3 = I3DUtil.getChildByName(itemAttachNode, "sleevesUnderGlove")
			elseif top.filename == "dataS/character/player02/clothes/tops/topHoodie.i3d" or top.filename == "dataS/character/player01/clothes/tops/topHoodie.i3d" then
				overwrite = I3DUtil.getChildByName(itemAttachNode, "Hoodie")
				sleeves1 = I3DUtil.getChildByName(itemAttachNode, "sleeveHands")
				sleeves2 = I3DUtil.getChildByName(itemAttachNode, "sleeveOverGlove")
				sleeves3 = I3DUtil.getChildByName(itemAttachNode, "sleeveUnderGlove")
			elseif top.filename == "dataS/character/player02/clothes/tops/topAviatorJacket.i3d" or top.filename == "dataS/character/player01/clothes/tops/topAviatorJacket.i3d" then
				overwrite = I3DUtil.getChildByName(itemAttachNode, "aviatorJacket")
				sleeves1 = I3DUtil.getChildByName(itemAttachNode, "sleevesHands")
				sleeves2 = I3DUtil.getChildByName(itemAttachNode, "sleeveOverGlove")
				sleeves3 = I3DUtil.getChildByName(itemAttachNode, "sleeveUnderGlove")
			else
				local materialId = getMaterial(itemAttachNode, 0)
				materialId = setMaterialDiffuseMapFromFile(materialId, PlayerStyle.FAMerchStore.diffuses[top.uvSlot], true, true, false)
				materialId = setMaterialCustomMapFromFile(materialId, "mMaskMap", PlayerStyle.FAMerchStore.masks[top.uvSlot], true, true, false)
				setMaterial(itemAttachNode, materialId, 0)
				updateColors(itemAttachNode, self.topConfig.items[self.topConfig.selection], self.topConfig.color)
			end

			if overwrite ~= "" then
				local materialId = getMaterial(overwrite, 0)
				materialId = setMaterialDiffuseMapFromFile(materialId, PlayerStyle.FAMerchStore.diffuses[top.uvSlot], true, true, false)
				materialId = setMaterialCustomMapFromFile(materialId, "mMaskMap", PlayerStyle.FAMerchStore.masks[top.uvSlot], true, true, false)
				setMaterial(overwrite, materialId, 0)
				setMaterial(sleeves1, materialId, 0)
				setMaterial(sleeves2, materialId, 0)
				setMaterial(sleeves3, materialId, 0)
				updateColors(overwrite, self.topConfig.items[self.topConfig.selection], self.topConfig.color)
				updateColors(itemAttachNode, self.topConfig.items[self.topConfig.selection], self.topConfig.color)
			end
		else
			updateColors(itemAttachNode, self.topConfig.items[self.topConfig.selection], self.topConfig.color)
		end
	end

	local bottomNode = nil

	if self.bottomConfig.selection ~= 0 then
		bottomNode = enable(self.bottomConfig.items[self.bottomConfig.selection], self.footwearConfig.selection ~= 0 and self.footwearConfig.items[self.footwearConfig.selection].extent or "low")

		updateColors(bottomNode, self.bottomConfig.items[self.bottomConfig.selection], self.bottomConfig.color)
	end

	if self.footwearConfig.selection ~= 0 then
		local extent = "high"

		if self.bottomConfig.selection ~= 0 then
			extent = self.bottomConfig.items[self.bottomConfig.selection].extent
		end

		if self.onepieceConfig.selection ~= 0 then
			extent = self.onepieceConfig.items[self.onepieceConfig.selection].extent
		end

		local item = self.footwearConfig.items[self.footwearConfig.selection]
		local itemAttachNode = enable(item, extent)

		updateColors(itemAttachNode, item, self.footwearConfig.color)
	end

	if self.glovesConfig.selection ~= 0 then
		local item = self.glovesConfig.items[self.glovesConfig.selection]
		local itemAttachNode = enable(item)

		updateColors(itemAttachNode, item, self.glovesConfig.color)
	end

	local hideGlasses = false

	if self.headgearConfig.selection ~= 0 then
		local item = self.headgearConfig.items[self.headgearConfig.selection]
		local itemAttachNode = enable(item)

		local headgear = self.headgearConfig.items[self.headgearConfig.selection]
		if headgear.uvSlot > 300 then
			local materialId = getMaterial(itemAttachNode, 0)
			materialId = setMaterialDiffuseMapFromFile(materialId, PlayerStyle.FAMerchStore.diffuses[headgear.uvSlot], true, true, false)
			materialId = setMaterialCustomMapFromFile(materialId, "mMaskMap", PlayerStyle.FAMerchStore.masks[headgear.uvSlot], true, true, false)
			setMaterial(itemAttachNode, materialId, 0)
		end
		updateColors(itemAttachNode, item, self.headgearConfig.color)

		if item.hideGlasses then
			hideGlasses = true
		end
	end

	if self.glassesConfig.selection ~= 0 and not hideGlasses then
		enable(self.glassesConfig.items[self.glassesConfig.selection])
	end

	if self.hairStyleConfig.selection ~= 0 then
		if (self.headgearConfig.selection ~= 0 or self.onepieceConfig.selection ~= 0 and self.onepieceConfig.items[self.onepieceConfig.selection].disabledOptions.headgear) and self.hatHairstyleIndex ~= nil then
			local item = self.hairStyleConfig.items[self.hatHairstyleIndex]
			local itemNode, itemNode2 = enable(item)

			updateHairColors(itemNode, item)
			updateHairColors(itemNode2, item)
		else
			local item = self.hairStyleConfig.items[self.hairStyleConfig.selection]
			local itemNode, itemNode2 = enable(item)

			updateHairColors(itemNode, item)
			updateHairColors(itemNode2, item)
		end
	end

	if self.facegearConfig.selection ~= 0 then
		enable(self.facegearConfig.items[self.facegearConfig.selection])
	else
		if self.mustacheConfig.selection ~= 0 then
			for i = self.mustacheConfig.selection, #self.mustacheConfig.items do
				local item = self.mustacheConfig.items[i]

				if item.face == nil or item.face == self.faceConfig.selection or self.faceConfig.selection == 0 and item.face == 1 then
					local itemNode, itemNode2 = enable(item)

					updateHairColors(itemNode, item)
					updateHairColors(itemNode2, item)

					break
				end
			end
		end

		if self.beardConfig.selection ~= 0 then
			for i = self.beardConfig.selection, #self.beardConfig.items do
				local item = self.beardConfig.items[i]

				if item.face == nil or item.face == self.faceConfig.selection or self.faceConfig.selection == 0 and item.face == 1 then
					local itemNode, itemNode2 = enable(item)

					updateHairColors(itemNode, item)
					updateHairColors(itemNode2, item)

					break
				end
			end
		end
	end

	if self.onepieceConfig.selection ~= 0 then
		local itemAttachNode = enable(self.onepieceConfig.items[self.onepieceConfig.selection], self.glovesConfig.selection ~= 0 and self.glovesConfig.items[self.glovesConfig.selection].extent or "hands", self.footwearConfig.selection ~= 0 and self.footwearConfig.items[self.footwearConfig.selection].extent or "low")

		updateColors(itemAttachNode, self.onepieceConfig.items[self.onepieceConfig.selection], self.onepieceConfig.color)
	end

	if (self.topConfig.selection ~= 0 or self.topConfig.selection == 0 and self.onepieceConfig.selection == 0) and self.bottomConfig.selection ~= 0 then
		local bottom = self.bottomConfig.items[self.bottomConfig.selection]
		local beltHidden = false

		if self.topConfig.selection ~= 0 then
			beltHidden = self.topConfig.items[self.topConfig.selection].beltHidden
		end

		if bottom.belt ~= nil then
			local node = I3DUtil.indexToObject(bottomNode, bottom.belt)

			setVisibility(node, not beltHidden)
		end
	end
end

PlayerStyle.apply = Utils.overwrittenFunction(PlayerStyle.apply, FAMerchStore.apply)



--Neue Kleidung in den Kleiderschrank einfügen
function FAMerchStore.loadClothing(self, superFunc, xmlFile, rootKey, itemKey, configName, isColorable, isHair, isFaceSpecific, isHeadgear, resetToOne)
	if itemKey == "top" or itemKey == "headgear" then
		local fileName = xmlFile:getFilename()
		local topsbefore = 0
		local headgearbefore = 0

		--Anzahl vorhandener Einträge auslesen
		xmlFile:iterate(rootKey .. ".top", function (_, key)
			topsbefore = topsbefore + 1
		end)
		xmlFile:iterate(rootKey .. ".headgear", function (_, key)
			headgearbefore = headgearbefore + 1
		end)

		for x = 301, PlayerStyle.FAMerchStore.maxUV do
			if fileName == "dataS/character/humans/player/player02.xml" and PlayerStyle.FAMerchStore.female[x] == true then
				if PlayerStyle.FAMerchStore.type[x] == "BALLCAP" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player02/attachments/headgearBallcap.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Ballcap")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 0 -.275")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "2")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.0183 0.0183 0.0183 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "BEANIE" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player02/attachments/headgearBeanie.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Beanie")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 .04 -.1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.0000 0.1845 0.2918 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "MILITIA" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player02/attachments/headgearMilitia.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Militia")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 0 -.25")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.045 0.055 0.085 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(0)#primary", "0.3 0.25 0.19 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(1)#primary", "0.21 0.17 0.12 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(2)#primary", "0.08 0.078 0.05 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(3)#primary", "0.045 0.055 0.085 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(4)#primary", "0.035 0.04 0.045 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(5)#primary", "0.025 0.02 0.02 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "VINTAGE" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player02/attachments/headgearVintage.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Vintage")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 0 -.2")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.075 0.08 0.12 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(0)#primary", "0.11 0.13 0.16 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(1)#primary", "0.16 0.13 0.11 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(2)#primary", "0.1 0.1 0.1 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(3)#primary", "0.12 0.08 0.075 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(4)#primary", "0.075 0.08 0.12 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(5)#primary", "0.05 0.05 0.05 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "TSHIRTF" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player02/clothes/tops/topTeeShirt.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_TeeShirt")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.3324 0.3324 0.3324 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "POLOSHIRT" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player02/clothes/tops/topCollaredShirt.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_PoloShirt")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "false")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.1683 0.1248 0.0999 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "HOODIE" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player02/clothes/tops/topHoodie.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Hoodie")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.2051 0.2051 0.2051 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(1)#name", "armsUpper")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(2)#name", "armsLower")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#node", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#type", "hands")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#node", "2")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#type", "over")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#node", "3")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#type", "under")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "SWEATER" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player02/clothes/tops/topLightSweater.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Sweater")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.0000 0.1845 0.2918 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(1)#name", "armsUpper")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(2)#name", "armsLower")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#node", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#type", "hands")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#node", "2")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#type", "over")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#node", "3")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#type", "under")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "AVIATOR" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player02/clothes/tops/topAviatorJacket.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_AviatorJacket")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.09 0.085 0.08 0.7")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(1)#name", "armsUpper")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(2)#name", "armsLower")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#node", "2")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#type", "hands")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#node", "3")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#type", "over")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#node", "4")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#type", "under")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(0)#primary", "0.155 0.14 0.14 0.25")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(1)#primary", "0.12 0.1 0.095 0.4")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(2)#primary", "0.1 0.095 0.111 0.55")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(3)#primary", "0.09 0.085 0.08 0.7")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(4)#primary", "0.08 0.0555 0.04 0.85")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(5)#primary", "0.04 0.035 0.035 1")
					topsbefore = topsbefore + 1
				end
			elseif fileName == "dataS/character/humans/player/player01.xml" and PlayerStyle.FAMerchStore.male[x] == true then
				if PlayerStyle.FAMerchStore.type[x] == "BALLCAP" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player01/attachments/headgearBallcap.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Ballcap")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 0 -.275")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "2")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.0183 0.0183 0.0183 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "BEANIE" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player01/attachments/headgearBeanie.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Beanie")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 .04 -.1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.0000 0.1845 0.2918 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "MILITIA" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player01/attachments/headgearMilitia.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Militia")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 0 -.25")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.045 0.055 0.085 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(0)#primary", "0.3 0.25 0.19 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(1)#primary", "0.21 0.17 0.12 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(2)#primary", "0.08 0.078 0.05 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(3)#primary", "0.045 0.055 0.085 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(4)#primary", "0.035 0.04 0.045 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(5)#primary", "0.025 0.02 0.02 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "VINTAGE" and itemKey == "headgear" then
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#filename", "dataS/character/player01/attachments/headgearVintage.i3d")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#node", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#attachPoint", "head")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Vintage")
					xmlFile:setFloat(rootKey .. ".headgear("..headgearbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#iconOffset", "0 0 -.2")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..")#defaultPrimary", "0.075 0.08 0.12 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(0)#primary", "0.11 0.13 0.16 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(1)#primary", "0.16 0.13 0.11 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(2)#primary", "0.1 0.1 0.1 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(3)#primary", "0.12 0.08 0.075 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(4)#primary", "0.075 0.08 0.12 1")
					xmlFile:setString(rootKey .. ".headgear("..headgearbefore..").colors.color(5)#primary", "0.05 0.05 0.05 1")
					headgearbefore = headgearbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "TSHIRTM" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player01/clothes/tops/topTeeShirt.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_TeeShirt")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.3324 0.3324 0.3324 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "POLOSHIRT" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player01/clothes/tops/topCollaredShirt.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_PoloShirt")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "false")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.1683 0.1248 0.0999 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "HOODIE" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player01/clothes/tops/topHoodie.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Hoodie")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.2051 0.2051 0.2051 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(1)#name", "armsUpper")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(2)#name", "armsLower")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#node", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#type", "hands")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#node", "2")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#type", "over")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#node", "3")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#type", "under")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "SWEATER" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player01/clothes/tops/topLightSweater.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_Sweater")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.0000 0.1845 0.2918 1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(1)#name", "armsUpper")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(2)#name", "armsLower")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#node", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#type", "hands")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#node", "2")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#type", "over")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#node", "3")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#type", "under")
					topsbefore = topsbefore + 1
				elseif PlayerStyle.FAMerchStore.type[x] == "AVIATOR" and itemKey == "top" then
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#filename", "dataS/character/player01/clothes/tops/topAviatorJacket.i3d")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#node", "1|0|0|0")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#attachPoint", "geo")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#name", PlayerStyle.FAMerchStore.modName[x].."_AviatorJacket")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#beltHidden", "true")
					xmlFile:setFloat(rootKey .. ".top("..topsbefore..")#uvSlot", x)
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#colorable", "1")
					xmlFile:setString(rootKey .. ".top("..topsbefore..")#defaultPrimary", "0.09 0.085 0.08 0.7")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(0)#name", "upperChest")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(1)#name", "armsUpper")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").hidesBodypart(2)#name", "armsLower")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#node", "2")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(0)#type", "hands")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#node", "3")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(1)#type", "over")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#node", "4")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").extent(2)#type", "under")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(0)#primary", "0.155 0.14 0.14 0.25")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(1)#primary", "0.12 0.1 0.095 0.4")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(2)#primary", "0.1 0.095 0.111 0.55")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(3)#primary", "0.09 0.085 0.08 0.7")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(4)#primary", "0.08 0.0555 0.04 0.85")
					xmlFile:setString(rootKey .. ".top("..topsbefore..").colors.color(5)#primary", "0.04 0.035 0.035 1")
					topsbefore = topsbefore + 1
				end
			end
		end
	end

	return superFunc(self, xmlFile, rootKey, itemKey, configName, isColorable, isHair, isFaceSpecific, isHeadgear, resetToOne)
end

PlayerStyle.loadClothing = Utils.overwrittenFunction(PlayerStyle.loadClothing, FAMerchStore.loadClothing)



--Übersichts-Bild austauschen
function FAMerchStore:populateCellForItemInSection(superFunc, list, section, index, cell)
	local itemIndex = self.indexMapping[section][index]
	local item = self.playerStyle[self.configName].items[itemIndex]
	local icon = cell:getAttribute("icon")
	local uvSlot = item.uvSlot
	if item.iconFilename ~= nil then
		icon:setImageFilename(item.iconFilename)
	else
		icon:setImageFilename(self.playerStyle.atlasFilename)

		if item.uvSlot > 300 then
			--UV 500 -> ein Bild
			--UV 501 -> zwei Bilder, links
			--UV 502 -> zwei Bilder, rechts
			uvSlot = 500
			if item.filename == "dataS/character/player01/clothes/tops/topCollaredShirt.i3d" or item.filename == "dataS/character/player01/clothes/tops/topLightSweater.i3d" or item.filename == "dataS/character/player01/clothes/tops/topHoodie.i3d" or item.filename == "dataS/character/player01/clothes/tops/topAviatorJacket.i3d" then
				uvSlot = 502
			elseif item.filename == "dataS/character/player02/clothes/tops/topCollaredShirt.i3d" or item.filename == "dataS/character/player02/clothes/tops/topLightSweater.i3d" or item.filename == "dataS/character/player02/clothes/tops/topHoodie.i3d" or item.filename == "dataS/character/player02/clothes/tops/topAviatorJacket.i3d" then
				uvSlot = 501
			end
			local filename = PlayerStyle.FAMerchStore.icons[item.uvSlot]
			icon:setImageFilename(filename)
		end
	end

	icon:setImageUVs(nil, self.playerStyle:getSlotUVs(uvSlot))
	cell:getAttribute("selected"):setVisible(self.savedPlayerStyle[self.configName].selection == itemIndex)
	cell:getAttribute("hasColors"):setVisible(item.numColors > 0)
end

WardrobeItemsFrame.populateCellForItemInSection = Utils.overwrittenFunction(WardrobeItemsFrame.populateCellForItemInSection, FAMerchStore.populateCellForItemInSection)



--UV-Slot-Nummer anpassen
function FAMerchStore:getSlotUVs(superFunc, slot)
	if slot == 0 or slot == nil then
		slot = 1
	end
	local rows = PlayerStyle.ATLAS_ROWS
	local columns = PlayerStyle.ATLAS_COLUMNS

	if slot > 300 then
		rows = 1
		if slot == 501 then
			slot = 0
			columns = 2
		elseif slot == 502 then
			slot = 1
			columns = 2
		else
			columns = 1
			slot = 0
		end
	else
		slot = slot - 1
	end

	local slotU = math.floor(slot / rows)
	local slotV = slot % rows
	local u = slotU / columns
	local v = slotV / rows
	local uvs = GuiUtils.getNormalizedValues(u .. " " .. v .. " " .. 1 / columns .. " " .. 1 / rows, {
		columns,
		rows
	})

	return uvs[1], 1 - uvs[2] - uvs[4], uvs[1], 1 - uvs[2], uvs[1] + uvs[3], 1 - uvs[2] - uvs[4], uvs[1] + uvs[3], 1 - uvs[2]
end

PlayerStyle.getSlotUVs = Utils.overwrittenFunction(PlayerStyle.getSlotUVs, FAMerchStore.getSlotUVs)



--PlayerStyle bei neuen Savegames erneut laden, damit die neue Kleidung vorhanden ist
function FAMerchStore:isValid(superFunc)
	if g_currentMission.missionInfo.isValid == false then
		if g_savegameXML ~= nil then
			local xmlFile = loadXMLFile("savegameXML", g_savegamePath)
			xmlFile = XMLFile.wrap(xmlFile)
			if xmlFile:hasProperty("gameSettings.lastPlayerStyle") then
				self:loadConfigurationXML(self.xmlFilename)
				self:loadFromXMLFile(xmlFile, "gameSettings.lastPlayerStyle")
			end
		end
	end

	return superFunc(self)
end

PlayerStyle.isValid = Utils.overwrittenFunction(PlayerStyle.isValid, FAMerchStore.isValid)

print("Wardrobe Plus loaded")